# This file is part of the lwbc cataloguing program
# Copyright © 2012 William Astle
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANT; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
package LWBC::Driver::Mongo;
use LWBC::Driver;
use Carp;
use strict;
use LWBC;
use MongoDB;
use Data::Dumper;

our @ISA = qw(LWBC::Driver);

sub new
{
	my ($class, %opts) = @_;
	my $self = $class->SUPER::new(%opts);
	bless ($self, $class);
	$self->{'_dbhost'} = 'localhost';
	$self->{'_dbport'} = undef;
	$self->{'_dbname'} = undef;
	$self->{'_dbhost'} = $opts{'dbhost'} if defined($opts{'dbhost'});
	$self->{'_dbport'} = $opts{'dbport'} if defined($opts{'dbport'});
	$self->{'_dbname'} = $opts{'dbname'} if defined($opts{'dbname'});
	croak "Must specify a database name" if !defined($self->{'_dbname'});
	my %copt;
	$copt{'host'} = $self->{'_dbhost'} if defined($self->{'_dbhost'});
	$copt{'port'} = $self->{'_dbport'} if defined($self->{'_dbport'});
	$self->{'_dbh'} = MongoDB::Connection->new(%copt);
	$self->{'_db'} = $self->{'_dbh'}->get_database($self->{'_dbname'});
	$self->{'_works'} = $self->{'_db'}->get_collection('works');
	$self->{'_people'} = $self->{'_db'}->get_collection('people');
	return $self;
}

sub fetch_work
{
	my ($self, $key) = @_;
	my $result = $self->{'_works'}->find_one([ '_id', $key]);
	
	my $ans = {};
	
	$ans->{'title'} = $result->{'title'};
	$ans->{'id'} = $result->{'_id'};
	$ans->{'copies'} = [];
	$ans->{'contribs'} = [];
	my $c = 0;
	my $i;
	foreach $i (@{ $result->{'copy'} })
	{
		$ans->{'copies'}->[$c] = { isbn => $i->{'isbn'}, lwid => $i->{'lwid'} };
		$c++;
	}
	
	$c = 0;
	foreach $i (@{ $result->{'author'} })
	{
		my $a = $self->fetch_person($i);
		my $sk = $a->{'sortkey'};
		$sk = $a->{'name'} if !defined($sk);
		
		$ans->{'contribs'}->[$c] = { 'type' => 'author', 'id' => $i, 'name' => $a->{'name'}, 'sortkey' => $sk };
		$c++;
	}
	
	foreach $i (@{ $result->{'editor'} })
	{
		my $a = $self->fetch_person($i);
		my $sk = $a->{'sortkey'};
		$sk = $a->{'name'} if !defined($sk);
		
		$ans->{'contribs'}->[$c] = { 'type' => 'editor', 'id' => $i, 'name' => $a->{'name'}, 'sortkey' => $sk };
		$c++;
	}
	
	return $ans;
}

sub fetch_person
{
	my ($self, $key) = @_;
	my $result = $self->{'_people'}->find_one([ '_id', $key]);
	my $sk = $result->{'sortkey'};
	$sk = $result->{'name'} unless defined($sk);
	my $ans = {
		'id' => $result->{'_id'},
		'name' => $result->{'name'},
		'sortkey' => $sk
	};
	return $ans;
}

sub save_work
{
	my ($self, $rec) = @_;
	my $recw = {};
	my $i;
	$recw->{'title'} = $rec->{'title'};
	my %tc = ();
	foreach $i ( @{ $rec->{'contribs'} })
	{
		my $tn = $i->{'type'};
		$tc{$tn} = 0 if (!defined($tc{$tn}));
		$recw->{$tn}->[$tc{$tn}] = $i->{'id'};
		$tc{$tn} += 1;
	}
	my $c = 0;
	foreach $i ( @{ $rec->{'copies'} })
	{
		$recw->{'copy'}->[$c] = { 'lwid' => $i->{'lwid'}, 'isbn' => $i->{'isbn'} };
		$c += 1;
	}
	$recw->{'_id'} = $rec->{'id'} if (defined($rec->{'id'}));

	return $self->{'_works'}->save($recw);
}

sub search_work
{
	my ($self, $search) = @_;
	
	my $result = $self->{'_works'}->find();
	$result->fields({ _id => 1});

	my $results = ();
	my $obj;
	
	while ($result->has_next)
	{
		$obj = $result->next;
		push @$results, $obj->{'_id'};
	}
	
	return $results;
}

sub add_person
{
	my ($self, $rec) = @_;
	my $res = $self->{'_people'}->insert($rec);
	return $res;
}

sub add_work
{
	my ($self, $rec) = @_;
	my $res = $self->{'_works'}->insert($rec);
	return $res;
}

sub search_person
{
	my ($self, $search) = @_;

	my $srch = {};
	
	if (defined($search->{'name'}))
	{
		my $t;
		$t = quotemeta($search->{'name'});
		$srch->{'name'} = qr/$t/i;
	}

	my $result = $self->{'_people'}->find($srch);
	$result->fields({ _id => 1});

	my $results = ();
	my $obj;
	
	while ($result->has_next)
	{
		$obj = $result->next;
		push @$results, $obj->{'_id'};
	}
	
	return $results;
}

1;
